import numpy as np

def primal_value(x, A, y, l1, l2):
    x = np.maximum(x,0)
    pv = 0.5 * np.linalg.norm(y-A@x,2)**2 + l1 * np.sum(x) + \
        (l2/2) * np.linalg.norm(x,2)**2
    return pv

def dual_value(u, A, y, l1, l2):
    dv = 0.5 * (np.linalg.norm(y,2)**2 - np.linalg.norm(y-u,2)**2 - \
        (1./l2) * np.linalg.norm(np.maximum(A.T @ u-l1,0),2)**2
    )
    return dv

def duality_gap(x, u, A, y, l1, l2):
    x = np.maximum(x,0)
    pv = primal_value(x, A, y, l1, l2)
    dv = dual_value(u, A, y, l1, l2)
    return np.abs(pv-dv)